@@ -52,7 +52,7 @@ New Config::
     evolution.obsdiscovery = yes
 
 Obsolescence Markers Discovery
-==============================
+------------------------------
 
 The evolve extension containts an experimental new protocol to discover common
 markers between local and remote repositories.
@@ -111,7 +111,7 @@ it back on)::
     evolution.obsdiscovery = no
 
 Effect Flag Experiment
-======================
+----------------------
 
 Evolve also records what changed between two evolutions of a changeset. For
 example, having this information is helpful to understand what changed between
@@ -153,8 +153,21 @@ Servers does not need to activate the effect flag recording. Effect flags that
 you create will not cause interference with other clients or servers without
 the effect flag recording.
 
+In-memory Evolve Experiment
+---------------------------
+
+The :hg:`evolve` command normally creates new changesets by writing the
+files to the working copy and then committing them from there. You can
+tell it to create the changesets without touching the working copy by
+setting this config::
+
+  [experimental]
+  evolution.in-memory = yes
+
+It will still update the working copy in case of conflicts.
+
 Template keywords
-=================
+-----------------
 
 Evolve provides one template keyword that helps explore obsolescence history:
 
@@ -175,6 +188,28 @@ template keywords from core Mercurial:
   - precursors (deprecated, use predecessors instead)
   - successors (deprecated, use successorssets instead)
   - troubles (deprecated, use instabilities instead)
+
+Revset predicates
+-----------------
+
+Evolve provides several revset predicates:
+
+  - unstable
+  - troubled (deprecated, use unstable instead)
+  - suspended
+  - predecessors
+  - precursors (deprecated, use predecessors instead)
+  - allpredecessors
+  - allprecursors (deprecated, use allpredecessors instead)
+  - successors
+  - allsuccessors
+
+Note that successors revset in evolve is not the same as successors revset in
+core Mercurial 4.3+. In evolve this revset returns only immediate successors,
+as opposed to all successors. Use "allsuccessors(set)" to obtain all
+successors.
+
+See :hg:`help revsets -v` for more information.
 """
 
 evolutionhelptext = b"""
@@ -241,22 +276,11 @@ or simply::
 
     [experimental]
     evolution=all
-""".strip()
+"""
 
 import sys
 
-try:
-    from mercurial import registrar
-    registrar.templatekeyword # new in hg-3.8
-except ImportError:
-    from . import metadata
-    raise ImportError(b'evolve needs Mercurial version %s or above' %
-                      min(metadata.testedwith.split()))
-
 import mercurial
-from mercurial import util
-
-from mercurial import obsolete
 
 from mercurial import (
     bookmarks as bookmarksmod,
@@ -267,8 +291,9 @@ from mercurial import (
     hg,
     lock as lockmod,
     node as nodemod,
+    obsolete,
     pycompat,
-    revset,
+    util,
 )
 
 from mercurial.i18n import _
@@ -287,6 +312,7 @@ from . import (
     obsexchange,
     obshashtree,
     obshistory,
+    revset,
     rewind,
     safeguard,
     templatekw,
@@ -337,17 +363,20 @@ eh.merge(compat.eh)
 eh.merge(cmdrewrite.eh)
 eh.merge(rewind.eh)
 eh.merge(headchecking.eh)
+eh.merge(revset.eh)
 uisetup = eh.finaluisetup
 extsetup = eh.finalextsetup
 reposetup = eh.finalreposetup
 cmdtable = eh.cmdtable
 configtable = eh.configtable
-revsetpredicate = eh.revsetpredicate
 templatekeyword = eh.templatekeyword
+revsetpredicate = eh.revsetpredicate
 
 # Configuration
 eh.configitem(b'experimental', b'evolutioncommands', [])
 eh.configitem(b'experimental', b'evolution.allnewcommands', None)
+eh.configitem(b'experimental', b'evolution.divergence-resolution-minimal', False)
+eh.configitem(b'experimental', b'evolution.in-memory', b'false')
 
 #####################################################################
 ### Option configuration                                          ###
@@ -401,12 +430,6 @@ def _configurecmdoptions(ui):
         for disabledcmd in set(cmdtable) - whitelist:
             del cmdtable[disabledcmd]
 
-#####################################################################
-### experimental behavior                                         ###
-#####################################################################
-
-getrevs = obsolete.getrevs
-
 #####################################################################
 ### Additional Utilities                                          ###
 #####################################################################
@@ -426,7 +449,8 @@ getrevs = obsolete.getrevs
 def setupparentcommand(ui):
 
     _alias, statuscmd = cmdutil.findcmd(b'status', commands.table)
-    pstatusopts = [o for o in statuscmd[1] if o[1] != b'rev']
+    inapplicable = {b'rev', b'change'}
+    pstatusopts = [o for o in statuscmd[1] if o[1] not in inapplicable]
 
     @eh.command(b'pstatus', pstatusopts,
                 **compat.helpcategorykwargs('CATEGORY_WORKING_DIRECTORY'))
@@ -442,7 +466,8 @@ def setupparentcommand(ui):
         return statuscmd[0](ui, repo, *args, **kwargs)
 
     _alias, diffcmd = cmdutil.findcmd(b'diff', commands.table)
-    pdiffopts = [o for o in diffcmd[1] if o[1] != b'rev']
+    inapplicable = {b'rev', b'from', b'to', b'change'}
+    pdiffopts = [o for o in diffcmd[1] if o[1] not in inapplicable]
 
     @eh.command(b'pdiff', pdiffopts,
                 **compat.helpcategorykwargs('CATEGORY_WORKING_DIRECTORY'))
@@ -464,179 +489,6 @@ def _installalias(ui):
                      b"diff --hidden --rev 'limit(predecessors(.),1)' --rev .",
                      b'evolve')
 
-### Unstable revset symbol
-
-@eh.revsetpredicate(b'unstable()')
-def revsetunstable(repo, subset, x):
-    """Changesets with instabilities.
-    """
-    revset.getargs(x, 0, 0, b'unstable takes no arguments')
-    troubled = set()
-    troubled.update(getrevs(repo, b'orphan'))
-    troubled.update(getrevs(repo, b'phasedivergent'))
-    troubled.update(getrevs(repo, b'contentdivergent'))
-    troubled = revset.baseset(troubled)
-    troubled.sort() # set is non-ordered, enforce order
-    return subset & troubled
-
-@eh.revsetpredicate(b'troubled()')    # legacy name
-def revsettroubled(repo, subset, x):
-    return revsetunstable(repo, subset, x)
-
-### Obsolescence graph
-
-# XXX SOME MAJOR CLEAN UP TO DO HERE XXX
-
-def _precursors(repo, s, includeidentical=False):
-    """Precursor of a changeset"""
-    cs = set()
-    getrev = compat.getgetrev(repo.changelog)
-    markerbysubj = repo.obsstore.predecessors
-    node = repo.changelog.node
-    for r in s:
-        for p in markerbysubj.get(node(r), ()):
-            if not includeidentical and p[2] & rewind.identicalflag:
-                continue
-            pr = getrev(p[0])
-            if pr is not None:
-                cs.add(pr)
-    cs -= repo.changelog.filteredrevs # nodemap has no filtering
-    return cs
-
-def _allprecursors(repo, s):  # XXX we need a better naming
-    """transitive precursors of a subset"""
-    node = repo.changelog.node
-    toproceed = [node(r) for r in s]
-    seen = set()
-    allsubjects = repo.obsstore.predecessors
-    while toproceed:
-        nc = toproceed.pop()
-        for mark in allsubjects.get(nc, ()):
-            np = mark[0]
-            if np not in seen:
-                seen.add(np)
-                toproceed.append(np)
-    getrev = compat.getgetrev(repo.changelog)
-    cs = set()
-    for p in seen:
-        pr = getrev(p)
-        if pr is not None:
-            cs.add(pr)
-    cs -= repo.changelog.filteredrevs # nodemap has no filtering
-    return cs
-
-def _successors(repo, s):
-    """Successors of a changeset"""
-    cs = set()
-    node = repo.changelog.node
-    getrev = compat.getgetrev(repo.changelog)
-    markerbyobj = repo.obsstore.successors
-    for r in s:
-        for p in markerbyobj.get(node(r), ()):
-            for sub in p[1]:
-                sr = getrev(sub)
-                if sr is not None:
-                    cs.add(sr)
-    cs -= repo.changelog.filteredrevs # nodemap has no filtering
-    return cs
-
-def _allsuccessors(repo, s, haltonflags=0):  # XXX we need a better naming
-    """transitive successors of a subset
-
-    haltonflags allows to provide flags which prevent the evaluation of a
-    marker.  """
-    node = repo.changelog.node
-    toproceed = [node(r) for r in s]
-    seen = set()
-    allobjects = repo.obsstore.successors
-    while toproceed:
-        nc = toproceed.pop()
-        for mark in allobjects.get(nc, ()):
-            if mark[2] & haltonflags:
-                continue
-            for sub in mark[1]:
-                if sub == nullid:
-                    continue # should not be here!
-                if sub not in seen:
-                    seen.add(sub)
-                    toproceed.append(sub)
-    getrev = compat.getgetrev(repo.changelog)
-    cs = set()
-    for s in seen:
-        sr = getrev(s)
-        if sr is not None:
-            cs.add(sr)
-    cs -= repo.changelog.filteredrevs # nodemap has no filtering
-    return cs
-
-#####################################################################
-### Extending revset and template                                 ###
-#####################################################################
-
-# this section add several useful revset symbol not yet in core.
-# they are subject to changes
-
-
-### XXX I'm not sure this revset is useful
-@eh.revsetpredicate(b'suspended()')
-def revsetsuspended(repo, subset, x):
-    """Obsolete changesets with non-obsolete descendants.
-    """
-    revset.getargs(x, 0, 0, b'suspended takes no arguments')
-    suspended = revset.baseset(getrevs(repo, b'suspended'))
-    suspended.sort()
-    return subset & suspended
-
-
-@eh.revsetpredicate(b'predecessors(set)')
-def revsetpredecessors(repo, subset, x):
-    """Immediate predecessors of changesets in set.
-    """
-    s = revset.getset(repo, revset.fullreposet(repo), x)
-    s = revset.baseset(_precursors(repo, s))
-    s.sort()
-    return subset & s
-
-
-@eh.revsetpredicate(b'precursors(set)')   # legacy name for predecessors
-def revsetprecursors(repo, subset, x):
-    return revsetpredecessors(repo, subset, x)
-
-
-@eh.revsetpredicate(b'allpredecessors(set)')
-def revsetallpredecessors(repo, subset, x):
-    """Transitive predecessors of changesets in set.
-    """
-    s = revset.getset(repo, revset.fullreposet(repo), x)
-    s = revset.baseset(_allprecursors(repo, s))
-    s.sort()
-    return subset & s
-
-
-@eh.revsetpredicate(b'allprecursors(set)')   # legacy name for allpredecessors
-def revsetallprecursors(repo, subset, x):
-    return revsetallpredecessors(repo, subset, x)
-
-
-@eh.revsetpredicate(b'successors(set)')
-def revsetsuccessors(repo, subset, x):
-    """Immediate successors of changesets in set.
-    """
-    s = revset.getset(repo, revset.fullreposet(repo), x)
-    s = revset.baseset(_successors(repo, s))
-    s.sort()
-    return subset & s
-
-@eh.revsetpredicate(b'allsuccessors(set)')
-def revsetallsuccessors(repo, subset, x):
-    """Transitive successors of changesets in set.
-    """
-    s = revset.getset(repo, revset.fullreposet(repo), x)
-    s = revset.baseset(_allsuccessors(repo, s))
-    s.sort()
-    return subset & s
-
-
 #####################################################################
 ### Various trouble warning                                       ###
 #####################################################################
@@ -1095,6 +947,10 @@ def _nextevolve(ui, repo, aspchildren, opts):
                                                              False))
     # making sure a next commit is formed
     if result[0] and result[1]:
+        # If using in-memory merge, _solveone() will not have updated the
+        # working copy, so we need to do that.
+        if evolvecmd.use_in_memory_merge(repo) and result[1]:
+            compat.update(repo[result[1]])
         ui.status(_(b'working directory is now at %s\n')
                   % ui.label(bytes(repo[b'.']), b'evolve.node'))
     return 0
