@@ -10,9 +10,11 @@ import array
 import contextlib
 
 from mercurial import (
+    cmdutil,
     context,
     copies,
     hg,
+    logcmdutil,
     merge as mergemod,
     obsolete,
     pycompat,
@@ -426,3 +428,16 @@ def cleanupnodes(repo, replacements, operation, moves=None, metadata=None):
 
     scmutil.cleanupnodes(repo, replacements=fixedreplacements, operation=operation,
                          moves=moves, metadata=metadata)
+
+if util.safehasattr(cmdutil, 'format_changeset_summary'):
+    def format_changeset_summary_fn(ui, repo, command, default_spec):
+        def show(ctx):
+            text = cmdutil.format_changeset_summary(ui, ctx, command=command,
+                                                    default_spec=default_spec)
+            ui.write(b'%s\n' % text)
+        return show
+else:
+    # hg <= 5.6 (96fcc37a9c80)
+    def format_changeset_summary_fn(ui, repo, command, default_spec):
+        return logcmdutil.changesetdisplayer(ui, repo,
+                                             {b'template': default_spec}).show
