@@ -28,7 +28,7 @@ from mercurial import (
     exchange,
     extensions,
     localrepo,
-    node,
+    node as nodemod,
     obsolete,
     scmutil,
     store,
@@ -195,7 +195,7 @@ def _queryrange(ui, repo, remote, allentries):
 def debugobshashrange(ui, repo, **opts):
     """display the ::REVS set topologically sorted in a stable way
     """
-    s = node.short
+    s = nodemod.short
     revs = scmutil.revrange(repo, opts['rev'])
     # prewarm depth cache
     if revs:
@@ -218,7 +218,7 @@ def debugobshashrange(ui, repo, **opts):
              r[1],
              rangelength(repo, r),
              depthrev(repo, r[0]),
-             node.short(_obshashrange(repo, r)))
+             s(_obshashrange(repo, r)))
         ui.status(linetemplate % d)
     repo.obsstore.rangeobshashcache.save(repo)
 
@@ -230,7 +230,7 @@ def _obshashrange(repo, rangeid):
     if obshash is not None:
         return obshash
     pieces = []
-    nullid = node.nullid
+    nullid = nodemod.nullid
     if repo.stablerange.rangelength(repo, rangeid) == 1:
         rangenode = cl.node(rangeid[0])
         tmarkers = repo.obsstore.relevantmarkers([rangenode])
@@ -249,7 +249,7 @@ def _obshashrange(repo, rangeid):
     # note: if there is only one subrange with actual data, we'll just
     # reuse the same hash.
     if not pieces:
-        obshash = node.nullid
+        obshash = nodemod.nullid
     elif len(pieces) != 1 or obshash is None:
         sha = hashlib.sha1()
         for p in pieces:
@@ -293,34 +293,35 @@ def _affectedby(repo, markers):
     """
     affected_nodes = set()
     known_markers = set(markers)
-    node_to_proceed = set()
-    marker_to_proceed = set(known_markers)
+    nodes_to_proceed = set()
+    markers_to_proceed = set(known_markers)
 
-    obsstore = repo.obsstore
+    successors = repo.obsstore.successors
+    predecessors = repo.obsstore.predecessors
 
-    while node_to_proceed or marker_to_proceed:
-        while marker_to_proceed:
-            m = marker_to_proceed.pop()
+    while nodes_to_proceed or markers_to_proceed:
+        while markers_to_proceed:
+            marker = markers_to_proceed.pop()
             # check successors and parent
-            if m[1]:
-                relevant = (m[1], )
+            if marker[1]:
+                relevant = (marker[1], )
             else: # prune case
-                relevant = ((m[0], ), m[5])
-            for l in relevant:
-                if l is None:
+                relevant = ((marker[0], ), marker[5])
+            for relnodes in relevant:
+                if relnodes is None:
                     continue
-                for n in l:
-                    if n not in affected_nodes:
-                        node_to_proceed.add(n)
-                    affected_nodes.add(n)
-        # marker_to_proceed is now empty:
-        if node_to_proceed:
-            n = node_to_proceed.pop()
+                for node in relnodes:
+                    if node not in affected_nodes:
+                        nodes_to_proceed.add(node)
+                    affected_nodes.add(node)
+        # markers_to_proceed is now empty:
+        if nodes_to_proceed:
+            node = nodes_to_proceed.pop()
             markers = set()
-            markers.update(obsstore.successors.get(n, ()))
-            markers.update(obsstore.predecessors.get(n, ()))
+            markers.update(successors.get(node, ()))
+            markers.update(predecessors.get(node, ()))
             markers -= known_markers
-            marker_to_proceed.update(markers)
+            markers_to_proceed.update(markers)
             known_markers.update(markers)
 
     return affected_nodes
@@ -448,7 +449,7 @@ class _obshashcache(obscache.dualsourcecache):
                                 self._data.pop(r, None)
                     except (sqlite3.DatabaseError, sqlite3.OperationalError) as exc:
                         repo.ui.log(b'evoext-cache',
-                                    b'error while updating obshashrange cache: %s'
+                                    b'error while updating obshashrange cache: %s\n'
                                     % forcebytestr(exc))
                         del self._updating
                         return
@@ -561,9 +562,9 @@ class _obshashcache(obscache.dualsourcecache):
                 del self._con
             self._new.clear()
             repo.ui.log(b'evoext-cache',
-                        b'error while saving new data: %s'
+                        b'error while saving new data: %s\n'
                         % forcebytestr(exc))
-            repo.ui.debug(b'evoext-cache: error while saving new data: %s'
+            repo.ui.debug(b'evoext-cache: error while saving new data: %s\n'
                           % forcebytestr(exc))
 
     def _trysave(self, repo):
@@ -575,7 +576,7 @@ class _obshashcache(obscache.dualsourcecache):
             con = self._db()
             if con is None:
                 repo.ui.log(b'evoext-cache', b'unable to write obshashrange cache'
-                            b' - cannot create database')
+                            b' - cannot create database\n')
                 return
             with con:
                 for req in _sqliteschema:
@@ -669,7 +670,7 @@ def _obshashrange_v0(repo, ranges):
     repo.obsstore.rangeobshashcache.update(repo)
     for r in ranges:
         if r[0] is None:
-            result.append(node.wdirid)
+            result.append(nodemod.wdirid)
         else:
             result.append(_obshashrange(repo, r))
     repo.obsstore.rangeobshashcache.save(repo)
@@ -843,13 +844,13 @@ def buildpullobsmarkersboundaries(pullop, bundle2=True):
     revs = unfi.revs(b'::(%ln - null) - %ld', pullop.common, filteredrevs)
     boundaries = {b'heads': pullop.pulledsubset}
     if not revs: # nothing common
-        boundaries[b'common'] = [node.nullid]
+        boundaries[b'common'] = [nodemod.nullid]
         return boundaries
 
     if not usediscovery(repo):
         # discovery disabled by users.
         repo.ui.status(obsdiscovery_skip_message)
-        boundaries[b'common'] = [node.nullid]
+        boundaries[b'common'] = [nodemod.nullid]
         return boundaries
 
     if bundle2 and _canobshashrange(repo, remote):
@@ -864,7 +865,7 @@ def buildpullobsmarkersboundaries(pullop, bundle2=True):
             common = repo.set("heads(only(%ld, %ln))", revs, missing)
             boundaries[b'common'] = [c.node() for c in common]
     else:
-        boundaries[b'common'] = [node.nullid]
+        boundaries[b'common'] = [nodemod.nullid]
     return boundaries
 
 # merge later for outer layer wrapping
