@@ -295,8 +295,8 @@ class stablerangesqlbase(stablerange.stablerangecached):
             if r'_con' in vars(self):
                 del self._con
             self._unsavedsubranges.clear()
-            repo.ui.log(b'evoext-cache', b'error while saving new data: %s' % forcebytestr(exc))
-            repo.ui.debug(b'evoext-cache: error while saving new data: %s' % forcebytestr(exc))
+            repo.ui.log(b'evoext-cache', b'error while saving new data: %s\n' % forcebytestr(exc))
+            repo.ui.debug(b'evoext-cache: error while saving new data: %s\n' % forcebytestr(exc))
 
     def _trysave(self, repo):
         repo = repo.unfiltered()
@@ -335,7 +335,7 @@ class stablerangesqlbase(stablerange.stablerangecached):
                 # to add. This will confuse sqlite
                 msg = _(b'stable-range cache: skipping write, '
                         b'database drifted under my feet\n')
-                hint = _(b'(disk: %s-%s vs mem: %s-%s)\n')
+                hint = _(b'(disk: %s-%d vs mem: %s-%d)\n')
                 data = (nodemod.hex(meta[2]), meta[1],
                         nodemod.hex(self._ondisktipnode), self._ondisktiprev)
                 repo.ui.warn(msg)
